using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class playerController : MonoBehaviour
{
    public float walkSpeed;
    public float jumpForce;
    bool pressedJump = false;
    Rigidbody playerRB;
    Collider playerCol;
    Vector3 playerSize;

    public AudioSource coinSound;

    public float cameraDistZ = 6f;

    float fallDistance = -1f;
    
    // Start is called before the first frame update
    void Start()
    {
        playerRB = GetComponent<Rigidbody>();
        playerCol = GetComponent<Collider>();
        playerSize = playerCol.bounds.size;
        cameraFollowPlayer();
    }

    // Update is called once per frame
    void FixedUpdate()
    {
        walkHandler();
        jumpHandler();
        cameraFollowPlayer();
        worldCheck();
    }
    
    //Walk Controller
    void walkHandler()
    {
    	float hAxis = Input.GetAxis("Horizontal");
        float vAxis = Input.GetAxis("Vertical");
        
        Vector3 movement = new Vector3(hAxis * walkSpeed * Time.deltaTime, 0, vAxis * walkSpeed * Time.deltaTime);
        
        Vector3 newPos = transform.position + movement;
        
        playerRB.MovePosition(newPos);

        if(hAxis != 0 || vAxis != 0)
        {
            Vector3 direction = new Vector3(hAxis, 0, vAxis);
            playerRB.rotation = Quaternion.LookRotation(direction);
        }
    }
    
    //Jump Controller
    void jumpHandler()
    {
    	float jAxis = Input.GetAxis("Jump");
    	
    	if(jAxis > 0)
    	{
    		bool isGrounded = checkGrounded();
    		
    		if(!pressedJump && isGrounded)
    		{
    			pressedJump = true;
    			Vector3 jumpVector = new Vector3(0, jAxis*jumpForce, 0);
    			playerRB.AddForce(jumpVector, ForceMode.VelocityChange);
    		}
    	}
    	else
    	{
    		pressedJump = false;
    	}
    }
    
    bool checkGrounded()
    {
    	Vector3 corner1 = transform.position + new Vector3(playerSize.x / 2, -playerSize.y / 2 + 0.01f, playerSize.z / 2);
    	Vector3 corner2 = transform.position + new Vector3(-playerSize.x / 2, -playerSize.y / 2 + 0.01f, playerSize.z / 2);
    	Vector3 corner3 = transform.position + new Vector3(playerSize.x / 2, -playerSize.y / 2 + 0.01f, -playerSize.z / 2);
    	Vector3 corner4 = transform.position + new Vector3(-playerSize.x / 2, -playerSize.y / 2 + 0.01f, -playerSize.z / 2);
    
    bool grounded1 = Physics.Raycast(corner1, -Vector3.up, 0.02f);
    bool grounded2 = Physics.Raycast(corner2, -Vector3.up, 0.02f);
    bool grounded3 = Physics.Raycast(corner3, -Vector3.up, 0.02f);
    bool grounded4 = Physics.Raycast(corner4, -Vector3.up, 0.02f);
    
    return (grounded1 || grounded2 || grounded3 || grounded4);
    }

    //Collider Coin, Enemy, Goal
    void OnTriggerEnter(Collider other)
    {
        if(other.CompareTag("Coin"))
        {
            Destroy(other.gameObject);
            coinSound.Play();
            managerGlobal.instance.increaseScore(1);
        }
        else if(other.CompareTag("Enemy"))
        {
            managerGlobal.instance.gameOver();
        }
        else if(other.CompareTag("Goal"))
        {
            managerGlobal.instance.increaseLevel();
        }
    }

    //Camera Controller
    void cameraFollowPlayer()
    {
        Vector3 cameraPos = Camera.main.transform.position;
        cameraPos.z = transform.position.z - cameraDistZ;
        Camera.main.transform.position = cameraPos;
    }

    //Level Fall Detection
    void worldCheck()
    {
        if(transform.position.y <= fallDistance)
        {
            managerGlobal.instance.gameOver();
        }
    }
    
}































